@@ -6,6 +6,7 @@ Synapse command line client
 For help, type synapse -h.
 
 """
+
 import argparse
 import os
 import shutil
@@ -18,134 +19,157 @@ import json
 
 
 def query(args, syn):
-    """
-    """
-    #TODO: Should use loop over multiple returned values if return is too long
-    results = syn.query(' '.join(args.queryString))
+    """TODO_Sphinx."""
+    
+    ## TODO: Should use loop over multiple returned values if return is too long
+    results = syn.chunkedQuery(' '.join(args.queryString))
 
-    results = results['results']
-    if len(results)==0:  #No results found
-        return
     headings = {}
-    for r in results:
-        for c in r:
-            headings[c] = True
+    temp = [] # Since query returns a generator, the results must be stored locally
+    for res in results:
+        temp.append(res)
+        for head in res:
+            headings[head] = True
+    if len(headings) == 0: # No results found
+        return 
     print '\t'.join(headings)
-    for result in results:
+    
+    for res in temp:
         out = []
-        for k in headings:
-            out.append(str(result.get(k, "")))
+        for key in headings:
+            out.append(str(res.get(key, "")))
         print "\t".join(out)
 
+        
 def get(args, syn):
-    """
+    """TODO_Sphinx."""
     
-    Arguments:
-    - `args`:
-    """
-    ent = syn.downloadEntity(args.id)
-    if 'files' in ent:
-        for f in ent['files']:
-            src = os.path.join(ent['cacheDir'], f)
-            dst = os.path.join('.', f.replace(".R_OBJECTS/",""))
-            sys.stderr.write('creating %s\n' %dst)
+    entity = syn.get(args.id)
+    
+    ## TODO: Is this part even necessary?
+    ## (Other than the print statements)
+    if 'files' in entity:
+        for file in entity['files']:
+            src = os.path.join(entity['cacheDir'], file)
+            dst = os.path.join('.', file.replace(".R_OBJECTS/",""))
+            print 'creating %s' % dst
             if not os.path.exists(os.path.dirname(dst)):
                 os.mkdir(dst)
             shutil.copyfile(src, dst)
     else:
         sys.stderr.write('WARNING: No files associated with entity %s\n' % (args.id,))
-        syn.printEntity(ent)
-    return ent
+        syn.printEntity(entity)
+    return entity
+    
+    
+def store(args, syn):
+    """TODO_Sphinx."""
+    
+    # Concatenate the multi-part arguments "name" and "description" 
+    # so that the other functions can accept them
+    if args.name is not None: args.name = ' '.join(args.name)
+    if args.description is not None: args.description = ' '.join(args.description)
+    
+    # --id indicates intention to update()
+    if args.id is not None:
+        if args.file is not None:
+            update(args, syn)
+        else:
+            print 'Update requires --file'
+        return
+        
+    # --file, --used, and --executed indicates intention to upload()
+    if args.file is not None or args.used is not None or args.executed is not None:
+        if args.parentid is not None:
+            upload(args, syn)
+        else: 
+            print 'Add requires --parentid'
+        return
+       
+    # --name indicates intention to create()
+    if args.name is not None:
+        if args.type is not None:
+            create(args, syn)
+        else:
+            print 'Create requires --type'
+        return
+        
+    print 'Could not interpret arguments.  Try using synapse create, add, or update.'
+
 
 def cat(args, syn):
-    """
+    """TODO_Sphinx."""
     
-    Arguments:
-    - `args`:
-    """
     signal.signal(signal.SIGPIPE, signal.SIG_DFL)
-    ent = syn.downloadEntity(args.id)
-    if 'files' in ent:
-        for f in ent['files']:
-            with open(os.path.join(ent['cacheDir'], f)) as fp:
-                for l in fp:
-                    sys.stdout.write(l)
+    entity = syn.get(args.id)
+    if 'files' in entity:
+        for file in entity['files']:
+            with open(os.path.join(entity['cacheDir'], file)) as input:
+                for line in input:
+                    print line
 
+                    
 def show(args, syn):
-    """
-    show metadata for an entity
-    """
-    ent = syn.getEntity(args.id)
+    """Show metadata for an entity."""
+    ent = syn.get(args.id, downloadFile=False)
     syn.printEntity(ent)
 
-def delete(args, syn):
-    """
     
-    Arguments:
-    - `args`:
-    """
-    syn.deleteEntity(args.id)
-    sys.stderr.write('Deleted entity: %s\n' % args.id)
+def delete(args, syn):
+    """TODO_Sphinx."""
+    
+    syn.delete(args.id)
+    print 'Deleted entity: %s' % args.id
 
     
 def upload(args, syn):
-    """
+    """TODO_Sphinx."""
     
-    Arguments:
-    - `args`:
-    """
     if args.type == 'File': args.type = 'FileEntity'
-    entity={'name': args.name,
-            'parentId': args.parentid,
-            'description':args.description,
-            'entityType': u'org.sagebionetworks.repo.model.%s' %args.type}
+    entity = {'name': args.name,
+              'parentId': args.parentid,
+              'description':args.description,
+              'entityType': u'org.sagebionetworks.repo.model.%s' % args.type, 
+              'path': args.file}
+    if utils.is_url(args.file):
+        entity['synapseStore'] = False
 
-    entity = syn.uploadFile(entity, args.file, used=args.used, executed=args.executed)
+    entity = syn.store(entity, used=args.used, executed=args.executed)
 
-    sys.stderr.write('Created entity: %s\t%s from file: %s\n' %(entity['id'], entity['name'], args.file))
+    print 'Created entity: %s\t%s from file: %s' %(entity['id'], entity['name'], args.file)
     return(entity)
 
 
 def create(args, syn):
-    """
-
-    Arguments:
-    - `args`:
-    """
+    """TODO_Sphinx."""
+    
     if args.type == 'File': args.type = 'FileEntity'
     entity={'name': args.name,
             'parentId': args.parentid,
             'description':args.description,
             'entityType': u'org.sagebionetworks.repo.model.%s' %args.type}
     entity=syn.createEntity(entity)
-    sys.stderr.write('Created entity: %s\t%s\n' %(entity['id'],entity['name']))
+    print 'Created entity: %s\t%s\n' %(entity['id'],entity['name'])
     return(entity)
 
 
 def update(args, syn):
-    """
+    """TODO_Sphinx."""
     
-    Arguments:
-    - `args`:
-    """
-    entity=syn.getEntity(args.id)
-    entity = syn.uploadFile(entity, args.file)
-    sys.stderr.write('Updated entity: %s\t%s from file: %s\n' %(entity['id'],entity['name'], args.file))
+    entity = syn.get(args.id)
+    entity.path = args.file
+    entity = syn.store(entity)
+    print 'Updated entity: %s\t%s from file: %s\n' %(entity['id'],entity['name'], args.file)
 
 
 def onweb(args, syn):
-    """
+    """TODO_Sphinx."""
     
-    Arguments:
-    - `args`:
-    """
     syn.onweb(args.id)
 
-
 def setProvenance(args, syn):
-    """
-    set provenance information on a synapse entity
-    """
+    """Set provenance information on a synapse entity."""
+    
     activity = Activity(name=args.name, description=args.description)
     if args.used:
         for item in args.used:
@@ -155,7 +179,7 @@ def setProvenance(args, syn):
             activity.used(item, wasExecuted=True)
     activity = syn.setProvenance(args.id, activity)
 
-    ## display the activity record, if -o or -output specified
+    # Display the activity record, if -o or -output specified
     if args.output:
         if args.output=='STDOUT':
             sys.stdout.write(json.dumps(activity))
@@ -165,19 +189,29 @@ def setProvenance(args, syn):
                 f.write(json.dumps(activity))
                 f.write('\n')
     else:
-        sys.stdout.write('Set provenance record %s on entity %s\n' % (str(activity['id']), str(args.id),))
+        print 'Set provenance record %s on entity %s\n' % (str(activity['id']), str(args.id))
 
 
 def getProvenance(args, syn):
+    """TODO_Sphinx."""
+    
     activity = syn.getProvenance(args.id)
 
     if args.output is None or args.output=='STDOUT':
-        sys.stdout.write(json.dumps(activity))
-        sys.stdout.write('\n')
+        print json.dumps(activity)
     else:
         with open(args.output, 'w') as f:
             f.write(json.dumps(activity))
             f.write('\n')
+    
+    
+def submit(args, syn):
+    """TODO_Sphinx."""
+    
+    if args.name is not None: args.name = ' '.join(args.name)
+    
+    submission = syn.submit(args.evaluation, args.entity, args.name)
+    print 'Submitted (id: %s) entity: %s\t%s to Evaluation: %s\n' %(submission['id'], submission['entityId'], submission['name'], submission['evaluationId'])
 
 
 def main():
@@ -207,6 +241,35 @@ def main():
                          help='Synapse ID of form syn123 of desired data object')
     parser_get.set_defaults(func=get)
 
+    parser_store = subparsers.add_parser('store', help='depending on the arguments supplied, store will either create, add, or update')
+    group = parser_store.add_mutually_exclusive_group()
+    group.add_argument('--id', metavar='syn123', type=str, 
+                         help='Synapse ID of form syn123 of the Synapse object to update')
+    group.add_argument('--parentid', metavar='syn123', type=str,  
+                         help='Synapse ID of project or folder where to upload new data.')
+    parser_store.add_argument('--name', type=str, nargs="+", 
+                         help='Name of data object in Synapse')
+    parser_store.add_argument('--description', type=str, nargs="+", 
+                         help='Description of data object in Synapse.')
+    parser_store.add_argument('--type', type=str, default='File',
+                         help='Type of object, such as "File", "Folder", or "Project", to create in Synapse. Defaults to "File"')
+    parser_store.add_argument('--used', metavar='TargetID', type=str, nargs='*',
+                         help='ID of a target data entity from which the specified entity is derived')
+    parser_store.add_argument('--executed', metavar='TargetID', type=str, nargs='*',
+                         help='ID of a code entity from which the specified entity is derived')
+    parser_store.add_argument('--file', type=str,
+                         help='file to be added to synapse.')
+    parser_store.set_defaults(func=store)
+    
+    parser_submit = subparsers.add_parser('submit', help='submit an entity for evaluation')
+    parser_submit.add_argument('--evaluation', type=str, required=True, 
+                         help='Evaluation ID where the entity will be submitted')
+    parser_submit.add_argument('--entity', type=str, required=True, 
+                         help='Synapse ID of the entity to be submitted')
+    parser_submit.add_argument('--name', type=str, nargs="+", 
+                         help='Name of the submission')
+    parser_submit.set_defaults(func=submit)
+
     parser_get = subparsers.add_parser('show', help='show metadata for an entity')
     parser_get.add_argument('id', metavar='syn123', type=str, 
                          help='Synapse ID of form syn123 of desired synapse object')
