@@ -1,8 +1,61 @@
+"""
+*****************
+Utility Functions
+*****************
+TODO_Sphinx (There's probably some way to generally describe these functions)
+   
+~~~~~~~~~~~~~
+File Handling
+~~~~~~~~~~~~~
+
+.. automethod:: synapseclient.utils.md5_for_file
+.. automethod:: synapseclient.utils.download_file
+.. automethod:: synapseclient.utils.extract_filename
+.. automethod:: synapseclient.utils.file_url_to_path
+.. automethod:: synapseclient.utils.normalize_whitespace
+
+~~~~~~~~~~~~~~~~~
+Property Juggling
+~~~~~~~~~~~~~~~~~
+
+.. automethod:: synapseclient.utils.guess_object_type
+.. automethod:: synapseclient.utils.id_of
+.. automethod:: synapseclient.utils.class_of
+.. automethod:: synapseclient.utils.get_properties
+.. automethod:: synapseclient.utils.get_entity_type
+.. automethod:: synapseclient.utils.is_url
+.. automethod:: synapseclient.utils.as_url
+.. automethod:: synapseclient.utils.is_synapse_entity
+.. automethod:: synapseclient.utils.is_synapse_id
+.. automethod:: synapseclient.utils.to_unix_epoch_time
+.. automethod:: synapseclient.utils.from_unix_epoch_time
+.. automethod:: synapseclient.utils.format_time_interval
+
+~~~~~~~~
+Chunking
+~~~~~~~~
+
+.. autoclass:: synapseclient.utils.Chunk
+.. automethod:: synapseclient.utils.chunks
+   
+~~~~~~~
+Testing
+~~~~~~~
+
+.. automethod:: synapseclient.utils.make_bogus_data_file
+.. automethod:: synapseclient.utils.make_bogus_binary_file
+.. automethod:: synapseclient.utils.synapse_error_msg
+.. automethod:: synapseclient.utils.debug_response
+.. automethod:: synapseclient.version_check.version_check
+
+"""
+
 #!/usr/bin/env python2.7
 
 # To debug this, python -m pdb myscript.py
 
 import os, sys, urllib, urlparse, hashlib, re
+import random
 import collections
 import tempfile
 import platform
@@ -11,16 +64,23 @@ from datetime import date as Date
 from numbers import Number
 
 UNIX_EPOCH = Datetime(1970, 1, 1, 0, 0)
+ISO_FORMAT = "%Y-%m-%dT%H:%M:%S.000Z"
 GB = 2**30
 MB = 2**20
 KB = 2**10
 
 
 def md5_for_file(filename, block_size=2**20):
-    '''
-    lifted this function from
-    http://stackoverflow.com/questions/1131220/get-md5-hash-of-a-files-without-open-it-in-python
-    '''
+    """
+    Calculates the MD5 of the given file.  See `source <http://stackoverflow.com/questions/1131220/get-md5-hash-of-a-files-without-open-it-in-python>`_.
+    
+    :param filename:   The file to read in
+    :param block_size: How much of the file to read in at once (bytes).
+                       Defaults to 1 MB
+    
+    :returns: The MD5
+    """
+    
     md5 = hashlib.md5()
     f = open(filename,'rb')
     while True:
@@ -31,10 +91,16 @@ def md5_for_file(filename, block_size=2**20):
     return(md5)
 
 
-## download a remote file
-## localFilePath can be None, in which case a temporary file is created
-## returns a tuple (localFilePath, HTTPmsg), see urllib.urlretrieve
 def download_file(url, localFilepath=None):
+    """
+    Downloads a remote file.
+    
+    :param localFilePath: May be None, in which case a temporary file is created
+    
+    :returns: A tuple (localFilePath, HTTPmsg).
+              See `urllib.urlretrieve <http://docs.python.org/2/library/urllib.html#urllib.urlretrieve>`_
+    """
+    
     if (localFilepath):
         dir = os.path.dirname(localFilepath)
         if not os.path.exists(dir):
@@ -42,15 +108,22 @@ def download_file(url, localFilepath=None):
     return urllib.urlretrieve(url, localFilepath)
 
 
-# this could be made more robust
-# see: http://tools.ietf.org/html/rfc6266
-# and the python library http://pypi.python.org/pypi/rfc6266
 def extract_filename(content_disposition):
+    """
+    TODO_Sphinx - This could be made more robust.
+    
+    See `this memo <http://tools.ietf.org/html/rfc6266>`_ 
+    and `this package <http://pypi.python.org/pypi/rfc6266>`_ 
+    for cryptic details.  (TODO_Sphinx - clarify this)
+    """
+    
     match = re.search('filename=([^ ]*)', content_disposition)
     return match.group(1) if match else 'filename'
 
 
 def guess_object_type(obj):
+    """Returns whether the given Synapse object is an Entity or Evaluation."""
+    
     if isinstance(obj, basestring):
         if obj.startswith('syn'):
             return 'entity'
@@ -63,6 +136,8 @@ def guess_object_type(obj):
 
 
 def _get_from_members_items_or_properties(obj, key):
+    """TODO_Sphinx."""
+    
     try:
         if hasattr(obj, key):
             return obj.id
@@ -77,10 +152,16 @@ def _get_from_members_items_or_properties(obj, key):
     except (KeyError, TypeError): pass
     return None
 
-#TODO: what does this do on an unsaved Synapse Entity object?
+## TODO: what does this do on an unsaved Synapse Entity object?
 def id_of(obj):
-    """Try to figure out the synapse ID of the given object. Accepted input
-    includes strings, entity objects, or entities represented by dictionaries"""
+    """
+    Try to figure out the Synapse ID of the given object.  
+    
+    :param obj: May be a string, Entity object, or dictionary
+    
+    :returns: The ID or throws an exception
+    """
+    
     if isinstance(obj, basestring):
         return obj
     if isinstance(obj, Number):
@@ -92,7 +173,8 @@ def id_of(obj):
 
 
 def class_of(obj):
-    """Return the class or type of the input object as a string"""
+    """Return the class or type of the input object as a string."""
+    
     if obj is None:
         return 'None'
     if hasattr(obj,'__class__'):
@@ -100,15 +182,20 @@ def class_of(obj):
     return str(type(obj))
 
 def get_properties(entity):
+    """Returns the dictionary of properties of the given Entity."""
+    
     return entity.properties if hasattr(entity, 'properties') else entity
 
 
 def get_entity_type(entity):
+    """Returns the Entity's type."""
+    
     return _get_from_members_items_or_properties(entity, 'entityType')
 
 
 def is_url(s):
-    """Return True if a string appears to be a valid URL."""
+    """Return True if the string appears to be a valid URL."""
+    
     if isinstance(s, basestring):
         try:
             url_parts = urlparse.urlsplit(s)
@@ -124,7 +211,8 @@ def is_url(s):
 
 
 def as_url(s):
-    """Try to convert input to a proper URL"""
+    """Tries to convert the input into a proper URL."""
+    
     url_parts = urlparse.urlsplit(s)
     ## Windows drive letter?
     if len(url_parts.scheme)==1 and url_parts.scheme.isalpha():
@@ -136,6 +224,8 @@ def as_url(s):
 
 
 def file_url_to_path(url, verify_exists=False):
+    """TODO_Sphinx"""
+    
     parts = urlparse.urlsplit(url)
     if parts.scheme=='file' or parts.scheme=='':
         path = parts.path
@@ -153,13 +243,16 @@ def file_url_to_path(url, verify_exists=False):
     return {}
 
 def is_synapse_entity(entity):
+    """TODO_Sphinx"""
+    
     if isinstance(entity, collections.Mapping):
         return 'entityType' in entity
     return False
 
 
 def is_synapse_id(obj):
-    """Returns a synapse ID, if the input is a synapse ID, otherwise returns None"""
+    """Returns None iff the input is a Synapse ID."""
+    
     if isinstance(obj, basestring):
         m = re.match(r'(syn\d+)', obj)
         if m:
@@ -167,16 +260,16 @@ def is_synapse_id(obj):
     return None
 
 def _is_date(dt):
-    """
-    Objects of class datetime.date and datetime.datetime will be recognized as dates
-    """
+    """TODO_Sphinx."""
+    
+    # Objects of class datetime.date and datetime.datetime will be recognized as dates
     return isinstance(dt,Date) or isinstance(dt,Datetime)
 
 
 def _to_list(value):
-    """
-    Convert the value (an iterable or a scalar value) to a list.
-    """
+    """TODO_Sphinx."""
+    
+    # Convert the value (an iterable or a scalar value) to a list.
     if isinstance(value, collections.Iterable) and not isinstance(value, basestring):
         return list(value)
     else:
@@ -184,9 +277,9 @@ def _to_list(value):
 
 
 def _to_iterable(value):
-    """
-    Convert the value (an iterable or a scalar value) to a list.
-    """
+    """TODO_Sphinx."""
+    
+    # Convert the value (an iterable or a scalar value) to a list.
     if isinstance(value, basestring):
         return (value,)
     if isinstance(value, collections.Iterable):
@@ -194,13 +287,19 @@ def _to_iterable(value):
     return (value,)
 
 
-def make_bogus_data_file(n=100, seed=12345):
-    """Make a bogus data file for testing. File will contain 'n'
-    random floating point numbers separated by commas. It is the
-    caller's responsibility to remove the file when finished.
+def make_bogus_data_file(n=100, seed=None):
     """
-    import random
-    random.seed(seed)
+    Makes a bogus data file for testing.  
+    It is the caller's responsibility to clean up the file when finished.
+    
+    :param n:    How many random floating point numbers to be written into the file, separated by commas
+    :param seed: Random seed for the random numbers
+    
+    :returns: The name of the file
+    """
+    
+    if seed is not None:
+        random.seed(seed)
     data = [random.gauss(mu=0.0, sigma=1.0) for i in range(n)]
 
     f = tempfile.NamedTemporaryFile(suffix=".txt", delete=False)
@@ -214,9 +313,16 @@ def make_bogus_data_file(n=100, seed=12345):
 
 
 def make_bogus_binary_file(n=1*MB, verbose=False):
-    """Make a bogus binary data file for testing. It is the
-    caller's responsibility to remove the file when finished.
     """
+    Makes a bogus binary data file for testing. 
+    It is the caller's responsibility to clean up the file when finished.
+    
+    :param n:       How many bytes to write
+    :param verbose: TODO_Sphinx
+    
+    :returns: The name of the file
+    """
+    
     if verbose:
         sys.stdout.write('writing bogus file')
     junk = os.urandom(min(n, 1*MB))
@@ -231,24 +337,23 @@ def make_bogus_binary_file(n=1*MB, verbose=False):
     return f.name
 
 
-## turns a datetime object into a unix epoch time expressed as a float
 def to_unix_epoch_time(dt):
     """
-    Convert either datetime.date or datetime.datetime objects to unix times
-    (milliseconds since midnight Jan 1, 1970)
+    Convert either `datetime.date or datetime.datetime objects 
+    <http://docs.python.org/2/library/datetime.html#available-types>`_ to UNIX time.
     """
+    
     if type(dt) == Date:
         return (dt - UNIX_EPOCH.date()).total_seconds() * 1000
     return (dt - UNIX_EPOCH).total_seconds() * 1000
 
 
 def from_unix_epoch_time(ms):
-    """
-    Return a datetime object given milliseconds since midnight Jan 1, 1970
-    """
-    ## utcfromtimestamp fails for negative values (dates before 1970-1-1) on windows
-    ## so, here's a hack that enables ancient events, such as Chris's birthday be
-    ## converted from ms since the unix epoch to higher level Datetime objects. Ha!
+    """Returns a Datetime object given milliseconds since midnight Jan 1, 1970."""
+    
+    # utcfromtimestamp() fails for negative values (dates before 1970-1-1) on Windows
+    # so, here's a hack that enables ancient events, such as Chris's birthday to be
+    # converted from milliseconds since the UNIX epoch to higher level Datetime objects. Ha!
     if platform.system()=='Windows' and ms < 0:
         mirror_date = Datetime.utcfromtimestamp(abs(ms)/1000.0)
         return (UNIX_EPOCH - (mirror_date-UNIX_EPOCH))
@@ -256,6 +361,8 @@ def from_unix_epoch_time(ms):
 
 
 def format_time_interval(seconds):
+    """TODO_Sphinx."""
+    
     periods = (
         ('year',        60*60*24*365),
         ('month',       60*60*24*30),
@@ -276,9 +383,9 @@ def format_time_interval(seconds):
     return ", ".join(result)
 
 
-## a helper method to find a particular used resource in an activity
-## that matches a predicate
 def _find_used(activity, predicate):
+    """Finds a particular used resource in an activity that matches a predicate."""
+    
     for resource in activity['used']:
         if predicate(resource):
             return resource
@@ -286,6 +393,9 @@ def _find_used(activity, predicate):
 
 
 def synapse_error_msg(ex):
+    """TODO_Sphinx."""
+    
+    
     if isinstance(ex, basestring):
         return ex
 
@@ -306,8 +416,10 @@ def synapse_error_msg(ex):
 
 def debug_response(response):
     """
-    Given a response object (from the requests library), print debugging information
+    Given a `requests.Response object <http://www.python-requests.org/en/latest/api/#requests.Response>`_, 
+    prints debugging information.
     """
+    
     try:
         print '\n\n'
         print '\nREQUEST ' + '>' * 52
@@ -332,9 +444,10 @@ def debug_response(response):
 BUFFER_SIZE = 8*KB
 
 class Chunk(object):
-    """
-    """
-    ## TODO implement seek and tell?
+    """TODO_Sphinx."""
+    
+    
+    ## TODO: implement seek and tell?
 
     def __init__(self, fileobj, size):
         self.fileobj = fileobj
@@ -343,6 +456,9 @@ class Chunk(object):
         self.closed = False
 
     def read(self, size=None):
+        """TODO_Sphinx."""
+    
+        
         if size is None or size <= 0:
             size = self.size - self.position
         else:
@@ -355,15 +471,23 @@ class Chunk(object):
         return self.fileobj.read(size)
 
     def mode(self):
+        """TODO_Sphinx."""
+    
         return self.fileobj.mode()
 
     def __len__(self):
+        """TODO_Sphinx."""
+    
         return self.size
 
     def __iter__(self):
+        """TODO_Sphinx."""
+    
         return self
 
     def next(self):
+        """TODO_Sphinx."""
+    
         if self.closed:
             raise StopIteration
         data = self.read(BUFFER_SIZE)
@@ -372,13 +496,14 @@ class Chunk(object):
         return data
 
     def close(self):
+        """TODO_Sphinx."""
+    
         self.closed = True
 
 
 def chunks(fileobj, chunksize=5*MB):
-    """
-    Generate file-like objects from which chunksize bytes can be streamed.
-    """
+    """Generate file-like objects from which chunksize bytes can be streamed."""
+    
     remaining = os.stat(fileobj.name).st_size
     while remaining > 0:
         chunk = Chunk(fileobj, size=min(remaining, chunksize))
@@ -386,9 +511,11 @@ def chunks(fileobj, chunksize=5*MB):
         yield chunk
 
 
-## http://code.activestate.com/recipes/576949/ (r3)
 def itersubclasses(cls, _seen=None):
     """
+    TODO_Sphinx - Clean up this comment
+    http://code.activestate.com/recipes/576949/ (r3)
+    
     itersubclasses(cls)
 
     Generator over all subclasses of a given class, in depth first order.
@@ -429,8 +556,11 @@ def itersubclasses(cls, _seen=None):
 
 
 def normalize_whitespace(s):
-    """Strip the string and replace all whitespace sequences and other
-    non-printable characters with a single space."""
+    """
+    Strips the string and replace all whitespace sequences and other
+    non-printable characters with a single space.
+    """
+    
     assert isinstance(s, str) or isinstance(s, unicode)
     return re.sub(r'[\x00-\x20\s]+', ' ', s.strip())
 
